#include "stdafx.h"

CQUBIOSettings :: CQUBIOSettings() { 
	m_iVersion=4;
	m_cConfigName.Set("");
	m_cAcqFileName.Set("Data%Y%m%d%H%M%S.qdf");
	m_iBufferSize=2000;
	m_iIORate=2000;
	m_cDataNotes.Set("Note\r\nCheck\r\nJunk");
	m_bScrollView=false;
	m_bTimeSep=false;
	m_bAutoNewData=true;
	}
	
bool CQUBIOSettings :: LoadFromFile( FILE * h ) { 
	BYTE btFlags;
	//-- Ver.1
	if( fread( &m_iVersion, sizeof(m_iVersion), 1, h ) != 1 
		|| ! FileLoadString( h, &m_cConfigName )
		|| ! FileLoadString( h, &m_cAcqFileName )
		|| fread( &m_iBufferSize, sizeof(m_iBufferSize), 1, h ) != 1 
		|| fread( &m_iIORate, sizeof(m_iIORate), 1, h ) != 1 
		|| ! FileLoadString( h, &m_cDataNotes )
		|| fread( &m_bScrollView, sizeof(m_bScrollView), 1, h ) != 1 
		|| fread( &btFlags, sizeof(BYTE), 1, h ) != 1 
		) 
		return false;
	m_bTimeSep = (btFlags&1)>0;
	m_bRecordDigout = (btFlags&8)>0;

	//-- ver. 2 Gain table inits
	if( m_iVersion>=2 ) { 
		if( fread( &m_bGainTable, sizeof(bool), 1, h) != 1 		
			|| ! FileLoadString( h, &m_cGainTable )
			|| fread( &m_iGainTableSource, sizeof(int), 1, h) != 1 		
			|| fread( &m_iGainTableApply, sizeof(int), 1, h) != 1 		
			)
		return false;
		}
	else { 
		m_bGainTable=false;
		m_cGainTable="";
		m_iGainTableSource=m_iGainTableApply=0;
		}
	//-- Ver. 3 Feedback inits
	if( m_iVersion>=3 ) { 
		if( fread( &m_bDefaultFeedback, sizeof(bool), 1, h) != 1 		
			|| ! FileLoadString( h, &m_cFeedbackName )
			|| fread( &m_iFeedbackSource, sizeof(int), 1, h) != 1 		
			|| fread( &m_iFeedbackApply, sizeof(int), 1, h) != 1 		
			)
		return false;
		}
	else { 
		m_bDefaultFeedback=false;
		m_cFeedbackName="";
		m_iFeedbackSource=m_iFeedbackApply=0;
		}

	if( m_iVersion>=4 ) {
		if( ! FileLoadString( h, &m_cSessionPath )
			|| ! FileLoadString( h, &m_cProtoPath )
			|| ! FileLoadString( h, &m_cDigiProtoPath )
			|| ! FileLoadString( h, &m_cDataPath ) )
		return false;
	//}
	//else {
	//	// blank strings by default?
	}

	if( m_iVersion>=5 ) {
		if ( fread( &m_bAutoNewData, sizeof(m_bAutoNewData), 1, h) != 1 )
			return false;
		while ( m_aQuickSessions.Count() )
			m_aQuickSessions.DeleteString(m_aQuickSessions.Count()-1);
		CString s;
		int nQuick = 0;
		if ( fread( &nQuick, sizeof(nQuick), 1, h) != 1 )
			return false;
		for (int i=0; i<nQuick; ++i)
			if ( ! FileLoadString( h, &s ) )
				return false;
			else
				m_aQuickSessions.AddString(s.ConstBuffer());
	}

	return true;
	}
	
bool CQUBIOSettings :: SaveToFile( FILE * h ) { 
	BYTE btFlags = (m_bTimeSep?1:0) + (m_bRecordDigout?8:0);
	
	m_iVersion=5;
	// note : any changes here should also be reflected in SaveSize
	bool rtn = fwrite( &m_iVersion, sizeof(m_iVersion), 1, h ) == 1 
		&& FileSaveString( h, m_cConfigName.Buffer() )
		&& FileSaveString( h, m_cAcqFileName.Buffer() )
		&& fwrite( &m_iBufferSize, sizeof(m_iBufferSize), 1, h ) == 1 
		&& fwrite( &m_iIORate, sizeof(m_iIORate), 1, h ) == 1 
		&& FileSaveString( h, m_cDataNotes.Buffer() )
		&& fwrite( &m_bScrollView, sizeof(m_bScrollView), 1, h ) == 1 
		&& fwrite( &btFlags, sizeof(BYTE), 1, h ) == 1 
		&& fwrite( &m_bGainTable, sizeof(bool), 1, h) == 1 		
		&& FileSaveString( h, m_cGainTable.Buffer() )
		&& fwrite( &m_iGainTableSource, sizeof(int), 1, h) == 1 		
		&& fwrite( &m_iGainTableApply, sizeof(int), 1, h) == 1 	
		&& fwrite( &m_bDefaultFeedback, sizeof(bool), 1, h) == 1 		
		&& FileSaveString( h, m_cFeedbackName.Buffer() )
		&& fwrite( &m_iFeedbackSource, sizeof(int), 1, h) == 1 		
		&& fwrite( &m_iFeedbackApply, sizeof(int), 1, h) == 1 	
		&& FileSaveString( h, m_cSessionPath.Buffer() )
		&& FileSaveString( h, m_cProtoPath.Buffer() )
		&& FileSaveString( h, m_cDigiProtoPath.Buffer() )
		&& FileSaveString( h, m_cDataPath.Buffer() )
		&& fwrite( &m_bAutoNewData, sizeof(m_bAutoNewData), 1, h) == 1
		;

	int nQuick = m_aQuickSessions.Count();
	rtn = rtn && fwrite( &nQuick, sizeof(nQuick), 1, h) == 1;
	for (int i=0; i<nQuick; ++i)
		rtn = rtn && FileSaveString( h, m_aQuickSessions.GetAt(i).ConstBuffer() );

	return rtn;
	}

int CQUBIOSettings :: SaveSize() { 
	int rtn = sizeof(m_iVersion)
		+ m_cConfigName.Length()+5
		+ m_cAcqFileName.Length()+5
		+ sizeof(m_iBufferSize)
		+ sizeof(m_iIORate)
		+ m_cDataNotes.Length()+5
		+ sizeof(m_bScrollView)
		+ sizeof(m_bTimeSep)
		+ sizeof(m_bGainTable)
		+ m_cGainTable.Length()+5
		+ sizeof(m_iGainTableSource)
		+ sizeof(m_iGainTableApply)
		+ sizeof(m_bDefaultFeedback)
		+ m_cFeedbackName.Length()+5
		+ sizeof(m_iFeedbackSource)
		+ sizeof(m_iFeedbackApply)
		+ m_cSessionPath.Length()+5
		+ m_cProtoPath.Length()+5
		+ m_cDigiProtoPath.Length()+5
		+ m_cDataPath.Length()+5
		+ sizeof(m_bAutoNewData)
		+ sizeof(int)
		;
	for (int i=0; i<m_aQuickSessions.Count(); ++i)
		rtn += m_aQuickSessions[i].Length() + 5;
	return rtn;
	}

void CQUBIOSettings::addSession(const char *sessName)
{
	int j = m_aQuickSessions.Find(sessName);
	if ( j == 0 )
		return; // already top session
	if ( j > 0 )
		m_aQuickSessions.DeleteString(j);
	else if ( m_aQuickSessions.Count() == 10 )
		m_aQuickSessions.DeleteString(9);
	m_aQuickSessions.AddString(sessName, 0);
}
